@@ -169,10 +169,6 @@ that is SSL-terminated.
     It is not advised to set this option on a Bokeh server directly facing
     the Internet.
 
-
-
-.. _userguide_cli_serve_session_id_options:
-
 Session ID Options
 ~~~~~~~~~~~~~~~~~~
 
@@ -294,7 +290,6 @@ import warnings
 
 from bokeh.application import Application
 from bokeh.resources import DEFAULT_SERVER_PORT
-from bokeh.server.server import Server
 from bokeh.util.logconfig import basicConfig
 from bokeh.util.string import nice_join, format_docstring
 from bokeh.settings import settings
@@ -304,7 +299,7 @@ from os import getpid
 from ..subcommand import Subcommand
 from ..util import build_single_handler_applications, die, report_server_init_errors
 
-LOGLEVELS = ('debug', 'info', 'warning', 'error', 'critical')
+LOGLEVELS = ('trace', 'debug', 'info', 'warning', 'error', 'critical')
 SESSION_ID_MODES = ('unsigned', 'signed', 'external-signed')
 DEFAULT_LOG_FORMAT = "%(asctime)s %(message)s"
 
@@ -365,9 +360,11 @@ class Serve(Subcommand):
 
     '''
 
+    #: name for this subcommand
     name = "serve"
 
     help = "Run a Bokeh server hosting one or more applications"
+
     args = base_serve_args + (
         ('files', dict(
             metavar='DIRECTORY-OR-SCRIPT',
@@ -471,6 +468,14 @@ class Serve(Subcommand):
 
 
     def invoke(self, args):
+        '''
+
+        '''
+
+        # protect this import inside a function so that "bokeh info" can work
+        # even if Tornado is not installed
+        from bokeh.server.server import Server
+
         argvs = { f : args.args for f in args.files}
         applications = build_single_handler_applications(args.files, argvs)
 
@@ -554,14 +559,13 @@ class Serve(Subcommand):
                         server.show(route)
                 server.io_loop.add_callback(show_callback)
 
-            address_string = ''
+            address_string = 'localhost'
             if server.address is not None and server.address != '':
-                address_string = ' address ' + server.address
+                address_string = server.address
 
-            log.info("Starting Bokeh server on port %d%s with applications at paths %r",
-                     server.port,
-                     address_string,
-                     sorted(applications.keys()))
+            for route in sorted(applications.keys()):
+                url = "http://%s:%d%s%s" % (address_string, server.port, server.prefix, route)
+                log.info("Bokeh app running at: %s" % url)
 
             log.info("Starting Bokeh server with process id: %d" % getpid())
             server.run_until_shutdown()
