@@ -59,7 +59,6 @@ from __future__ import absolute_import
 from six import string_types
 
 from .. import colors, palettes
-from ..util.deprecation import deprecated
 
 class Enumeration(object):
     ''' Represent an enumerated collection of values.
@@ -128,6 +127,9 @@ def enumeration(*values, **kwargs):
 
     return type("Enumeration", (Enumeration,), attrs)()
 
+#: Specify whether events should be combined or collected as-is when a Document hold is in effect
+HoldPolicy = enumeration( "combine", "collect")
+
 #: Specify whether a dimension or coordinate is latitude or longitude
 LatLon = enumeration("lat", "lon")
 
@@ -177,21 +179,6 @@ LegendLocation = Anchor = enumeration(
     "center_left", "center",        "center_right",
     "bottom_left", "bottom_center", "bottom_right")
 
-#: Deprecated legend location/anchor
-DeprecatedLegendLocation = DeprecatedAnchor = enumeration("left_center", "right_center")
-def accept_left_right_center(value):
-    ''' Accept and convert deprecated location values.
-
-    NOT FOR GENERAL USE
-
-    This is a temporary function to support a deprecation, and will be removed
-    when the deprecation is completed.
-
-    '''
-    deprecated((0, 12, 4), "'left_center' and 'right_center' enumerations",
-                           "'center_left' or 'center_right' respectively")
-    return {"left_center": "center_left", "right_center": "center_right"}[value]
-
 #: Specify a location in plot layouts
 Location = enumeration("above", "below", "left", "right")
 
@@ -208,7 +195,7 @@ DashPattern = enumeration("solid", "dashed", "dotted", "dotdash", "dashdot")
 ButtonType = enumeration("default", "primary", "success", "warning", "danger", "link")
 
 #: Specify one of the 137 named CSS colors
-NamedColor = enumeration(*colors.__colors__, case_sensitive=False)
+NamedColor = enumeration(*colors.named.__all__, case_sensitive=False)
 
 #: Specify the name of a palette from :ref:`bokeh.palettes`
 Palette = enumeration(*palettes.__palettes__)
@@ -218,7 +205,7 @@ MapType = enumeration("satellite", "roadmap", "terrain", "hybrid")
 
 #: Specify a format for printing dates
 DateFormat = enumeration("ATOM", "W3C", "RFC-3339", "ISO-8601", "COOKIE", "RFC-822",
-                         "RFC-850", "RFC-1036", "RFC-1123", "RFC-2822", "RSS", "TICKS", "TIMESTAMP")
+                         "RFC-850", "RFC-1036", "RFC-1123", "RFC-2822", "RSS", "TIMESTAMP")
 
 #: Specify a policy for  how numbers should be rounded
 RoundingFunction = enumeration("round", "nearest", "floor", "rounddown", "ceil", "roundup")
@@ -238,11 +225,6 @@ RenderLevel = enumeration("image", "underlay", "glyph", "annotation", "overlay")
 #: Specify a render mode for renderers that support both Canvas or CSS rendering
 RenderMode = enumeration("canvas", "css")
 
-# TODO (bev) Aggregation can probably be deprecated and removed
-
-#: Specify an aggregation type
-Aggregation = enumeration("sum", "mean", "count", "nunique", "median", "min", "max")
-
 #: Specify a start/end value
 StartEnd = enumeration("start", "end")
 
