@@ -104,14 +104,16 @@ import re
 
 from six import string_types, iteritems
 
-from ..colors import RGB
+from .. import colors
 from ..util.dependencies import import_optional
 from ..util.serialization import convert_datetime_type, decode_base64_dict, transform_column_source_data
 from ..util.string import nice_join, format_docstring
 
 from .property.bases import ContainerProperty, DeserializationError, ParameterizedProperty, Property, PrimitiveProperty
+from .property.containers import PropertyValueColumnData, PropertyValueDict, PropertyValueList
 from .property.descriptor_factory import PropertyDescriptorFactory
-from .property.descriptors import BasicPropertyDescriptor, DataSpecPropertyDescriptor, UnitsSpecPropertyDescriptor
+from .property.descriptors import (BasicPropertyDescriptor, ColumnDataPropertyDescriptor, DataSpecPropertyDescriptor,
+                                   UnitsSpecPropertyDescriptor)
 from . import enums
 
 pd = import_optional('pandas')
@@ -761,6 +763,17 @@ class Auto(Enum):
     def _sphinx_type(self):
         return self._sphinx_prop_link()
 
+class RGB(Property):
+    ''' Accept Date (but not DateTime) values.
+
+    '''
+
+    def validate(self, value):
+        super(RGB, self).validate(value)
+
+        if not (value is None or isinstance(value, colors.RGB)):
+            raise ValueError("expected RGB value, got %r" % (value,))
+
 # Properties useful for defining visual attributes
 class Color(Either):
     ''' Accept color values in a variety of ways.
@@ -802,7 +815,8 @@ class Color(Either):
         types = (Enum(enums.NamedColor),
                  Regex("^#[0-9a-fA-F]{6}$"),
                  Tuple(Byte, Byte, Byte),
-                 Tuple(Byte, Byte, Byte, Percent))
+                 Tuple(Byte, Byte, Byte, Percent),
+                 RGB)
         super(Color, self).__init__(*types, default=default, help=help)
 
     def __str__(self):
@@ -810,7 +824,7 @@ class Color(Either):
 
     def transform(self, value):
         if isinstance(value, tuple):
-            value = RGB(*value).to_css()
+            value = colors.RGB(*value).to_css()
         return value
 
     def _sphinx_type(self):
@@ -1035,7 +1049,7 @@ class Date(Property):
     ''' Accept Date (but not DateTime) values.
 
     '''
-    def __init__(self, default=datetime.date.today(), help=None):
+    def __init__(self, default=None, help=None):
         super(Date, self).__init__(default=default, help=help)
 
     def transform(self, value):
@@ -1203,6 +1217,19 @@ class List(Seq):
         # optional values. Also in Dict.
         super(List, self).__init__(item_type, default=default, help=help)
 
+    @classmethod
+    def wrap(cls, value):
+        ''' Some property types need to wrap their values in special containers, etc.
+
+        '''
+        if isinstance(value, list):
+            if isinstance(value, PropertyValueList):
+                return value
+            else:
+                return PropertyValueList(value)
+        else:
+            return value
+
     @classmethod
     def _is_seq(cls, value):
         return isinstance(value, list)
@@ -1258,10 +1285,22 @@ class Dict(ContainerProperty):
                     all(self.keys_type.is_valid(key) and self.values_type.is_valid(val) for key, val in iteritems(value))):
                 raise ValueError("expected an element of %s, got %r" % (self, value))
 
+    @classmethod
+    def wrap(cls, value):
+        ''' Some property types need to wrap their values in special containers, etc.
+
+        '''
+        if isinstance(value, dict):
+            if isinstance(value, PropertyValueDict):
+                return value
+            else:
+                return PropertyValueDict(value)
+        else:
+            return value
+
     def _sphinx_type(self):
         return self._sphinx_prop_link() + "( %s, %s )" % (self.keys_type._sphinx_type(), self.values_type._sphinx_type())
 
-
 class ColumnData(Dict):
     ''' Accept a Python dictionary suitable as the ``data`` attribute of a
     :class:`~bokeh.models.sources.ColumnDataSource`.
@@ -1271,6 +1310,23 @@ class ColumnData(Dict):
 
     '''
 
+    def make_descriptors(self, base_name):
+        ''' Return a list of ``ColumnDataPropertyDescriptor`` instances to
+        install on a class, in order to delegate attribute access to this
+        property.
+
+        Args:
+            base_name (str) : the name of the property these descriptors are for
+
+        Returns:
+            list[ColumnDataPropertyDescriptor]
+
+        The descriptors returned are collected by the ``MetaHasProps``
+        metaclass and added to ``HasProps`` subclasses during class creation.
+        '''
+        return [ ColumnDataPropertyDescriptor(base_name, self) ]
+
+
     def from_json(self, json, models=None):
         ''' Decodes column source data encoded as lists or base64 strings.
         '''
@@ -1296,10 +1352,22 @@ class ColumnData(Dict):
                 new_data[key] = self.values_type.from_json(value, models)
         return new_data
 
-
     def serialize_value(self, value):
         return transform_column_source_data(value)
 
+    @classmethod
+    def wrap(cls, value):
+        ''' Some property types need to wrap their values in special containers, etc.
+
+        '''
+        if isinstance(value, dict):
+            if isinstance(value, PropertyValueColumnData):
+                return value
+            else:
+                return PropertyValueColumnData(value)
+        else:
+            return value
+
 class Tuple(ContainerProperty):
     ''' Accept Python tuple values.
 
@@ -1443,6 +1511,7 @@ class DataSpec(Either):
                 Either(
                     String,
                     Instance('bokeh.models.transforms.Transform'),
+                    Instance('bokeh.models.expressions.Expression'),
                     value_type)),
             value_type,
             default=default,
@@ -1491,7 +1560,7 @@ class DataSpec(Either):
     def _sphinx_type(self):
         return self._sphinx_prop_link()
 
-_FieldValueTransform = Enum("field", "value", "transform")
+_ExprFieldValueTransform = Enum("expr", "field", "value", "transform")
 
 class NumberSpec(DataSpec):
     ''' A |DataSpec| property that accepts numeric and datetime fixed values.
@@ -1509,7 +1578,7 @@ class NumberSpec(DataSpec):
         m.location = "foo" # field
 
     '''
-    def __init__(self, default=None, help=None, key_type=_FieldValueTransform, accept_datetime=True):
+    def __init__(self, default=None, help=None, key_type=_ExprFieldValueTransform, accept_datetime=True):
         super(NumberSpec, self).__init__(key_type, Float, default=default, help=help)
         self.accepts(TimeDelta, convert_datetime_type)
         if accept_datetime:
@@ -1531,7 +1600,7 @@ class StringSpec(DataSpec):
         m.title = "foo"        # field
 
     '''
-    def __init__(self, default, help=None, key_type=_FieldValueTransform):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
         super(StringSpec, self).__init__(key_type, List(String), default=default, help=help)
 
     def prepare_value(self, cls, name, value):
@@ -1563,7 +1632,7 @@ class FontSizeSpec(DataSpec):
     '''
     _font_size_re = re.compile(r"^[0-9]+(.[0-9]+)?(%|em|ex|ch|ic|rem|vw|vh|vi|vb|vmin|vmax|cm|mm|q|in|pc|pt|px)$", re.I)
 
-    def __init__(self, default, help=None, key_type=_FieldValueTransform):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
         super(FontSizeSpec, self).__init__(key_type, List(String), default=default, help=help)
 
     def prepare_value(self, cls, name, value):
@@ -1583,7 +1652,7 @@ class FontSizeSpec(DataSpec):
             elif value[0].isdigit() and self._font_size_re.match(value) is None:
                 raise ValueError("%r is not a valid font size value" % value)
 
-_FieldValueTransformUnits = Enum("field", "value", "transform", "units")
+_ExprFieldValueTransformUnits = Enum("expr", "field", "value", "transform", "units")
 
 class UnitsSpec(NumberSpec):
     ''' A |DataSpec| property that accepts numeric fixed values, and also
@@ -1591,7 +1660,7 @@ class UnitsSpec(NumberSpec):
 
     '''
     def __init__(self, default, units_type, units_default, help=None):
-        super(UnitsSpec, self).__init__(default=default, help=help, key_type=_FieldValueTransformUnits)
+        super(UnitsSpec, self).__init__(default=default, help=help, key_type=_ExprFieldValueTransformUnits)
         self._units_type = self._validate_type_param(units_type)
         # this is a hack because we already constructed units_type
         self._units_type.validate(units_default)
@@ -1731,7 +1800,7 @@ class ColorSpec(DataSpec):
         m.color = field("firebrick")       # field (named "firebrick")
 
     '''
-    def __init__(self, default, help=None, key_type=_FieldValueTransform):
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
         super(ColorSpec, self).__init__(key_type, Color, default=default, help=help)
 
     @classmethod
@@ -1760,10 +1829,17 @@ class ColorSpec(DataSpec):
 
         # Check for RGB or RGBa tuple
         if isinstance(val, tuple):
-            return dict(value=RGB(*val).to_css())
+            return dict(value=colors.RGB(*val).to_css())
 
         # Check for data source field name
+        if isinstance(val, colors.RGB):
+            return val.to_css()
+
+        # Check for data source field name or rgb(a) string
         if isinstance(val, string_types):
+            if val.startswith(("rgb(", "rgba(")):
+                return val
+
             return dict(field=val)
 
         # Must be dict, return new dict
@@ -1787,7 +1863,30 @@ class ColorSpec(DataSpec):
 # DataSpec helpers
 #------------------------------------------------------------------------------
 
-def field(name):
+def expr(expression, transform=None):
+    ''' Convenience function to explicitly return an "expr" specification for
+    a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
+
+    Args:
+        expression (Expression) : a computed expression for a
+            ``DataSpec`` property.
+
+        transform (Transform, optional) : a transform to apply (default: None)
+
+    Returns:
+        dict : ``{ "expr": expression }``
+
+    .. note::
+        This function is included for completeness. String values for
+        property specifications are by default interpreted as field names.
+
+    '''
+    if transform:
+        return dict(expr=expression, transform=transform)
+    return dict(expr=expression)
+
+
+def field(name, transform=None):
     ''' Convenience function to explicitly return a "field" specification for
     a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
 
@@ -1795,6 +1894,8 @@ def field(name):
         name (str) : name of a data source field to reference for a
             ``DataSpec`` property.
 
+        transform (Transform, optional) : a transform to apply (default: None)
+
     Returns:
         dict : ``{ "field": name }``
 
@@ -1803,15 +1904,19 @@ def field(name):
         property specifications are by default interpreted as field names.
 
     '''
+    if transform:
+        return dict(field=name, transform=transform)
     return dict(field=name)
 
-def value(val):
+def value(val, transform=None):
     ''' Convenience function to explicitly return a "value" specification for
     a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
 
     Args:
         val (any) : a fixed value to specify for a ``DataSpec`` property.
 
+        transform (Transform, optional) : a transform to apply (default: None)
+
     Returns:
         dict : ``{ "value": name }``
 
@@ -1830,6 +1935,8 @@ def value(val):
                    text_font_size=value("12pt"), source=source)
 
     '''
+    if transform:
+        return dict(value=val, transform=transform)
     return dict(value=val)
 
 #------------------------------------------------------------------------------
