@@ -9,10 +9,8 @@
 .. bokeh-jinja:: bokeh.core.templates.FILE
 .. bokeh-jinja:: bokeh.core.templates.JS_RESOURCES
 .. bokeh-jinja:: bokeh.core.templates.NOTEBOOK_LOAD
-.. bokeh-jinja:: bokeh.core.templates.NOTEBOOK_DIV
 .. bokeh-jinja:: bokeh.core.templates.PLOT_DIV
 .. bokeh-jinja:: bokeh.core.templates.SCRIPT_TAG
-.. bokeh-jinja:: bokeh.core.templates.NOTEBOOK_CELL_OBSERVER
 
 '''
 from __future__ import absolute_import
@@ -38,12 +36,8 @@ FILE = _env.get_template("file.html")
 
 NOTEBOOK_LOAD = _env.get_template("notebook_load.html")
 
-NOTEBOOK_DIV = _env.get_template("notebook_div.html")
-
 AUTOLOAD_JS = _env.get_template("autoload_js.js")
 
 AUTOLOAD_NB_JS = _env.get_template("autoload_nb_js.js")
 
 AUTOLOAD_TAG = _env.get_template("autoload_tag.html")
-
-NOTEBOOK_CELL_OBSERVER = _env.get_template("notebook_cell_observer.js")
