@@ -6,11 +6,10 @@ from __future__ import absolute_import
 
 from six import string_types
 
-from ..core.enums import (accept_left_right_center, AngleUnits, DeprecatedLegendLocation, Dimension,
-                          FontStyle, LegendClickPolicy, LegendLocation, Orientation, RenderMode,
-                          SpatialUnits, TextAlign)
+from ..core.enums import (AngleUnits, Dimension, FontStyle, LegendClickPolicy, LegendLocation,
+                          Orientation, RenderMode, SpatialUnits, TextAlign)
 from ..core.has_props import abstract
-from ..core.properties import (Angle, AngleSpec, Auto, Bool, ColorSpec, Datetime, DistanceSpec, Either,
+from ..core.properties import (Angle, AngleSpec, Auto, Bool, ColorSpec, Datetime, Dict, DistanceSpec, Either,
                                Enum, Float, FontSizeSpec, Include, Instance, Int, List, NumberSpec, Override,
                                Seq, String, StringSpec, Tuple, value)
 from ..core.property_mixins import FillProps, LineProps, TextProps
@@ -89,7 +88,7 @@ class Legend(Annotation):
     ``bokeh.core.enums.LegendLocation``'s enumerated values, or a ``(x, y)``
     tuple indicating an absolute location absolute location in screen
     coordinates (pixels from the bottom-left corner).
-    """).accepts(Enum(DeprecatedLegendLocation), accept_left_right_center)
+    """)
 
     orientation = Enum(Orientation, default="vertical", help="""
     Whether the legend entries should be placed vertically or horizontally
@@ -157,7 +156,8 @@ class Legend(Annotation):
     """)
 
     padding = Int(10, help="""
-    Amount of padding around the contents of the legend.
+    Amount of padding around the contents of the legend. Only applicable when
+    when border is visible, otherwise collapses to 0.
     """)
 
     spacing = Int(3, help="""
@@ -249,6 +249,11 @@ class ColorBar(Annotation):
     A TickFormatter to use for formatting the visual appearance of ticks.
     """)
 
+    major_label_overrides = Dict(Either(Float, String), String, default={}, help="""
+    Provide explicit tick label values for specific tick locations that
+    override normal formatting.
+    """)
+
     color_mapper = Instance(ContinuousColorMapper, help="""
     A continuous color mapper containing a color palette to render.
 
