@@ -9,6 +9,7 @@ from ..util.dependencies import import_optional
 from ..util.warnings import BokehUserWarning
 
 from .callbacks import Callback
+from .filters import Filter
 
 pd = import_optional('pandas')
 
@@ -69,16 +70,61 @@ class ColumnarDataSource(DataSource):
 class ColumnDataSource(ColumnarDataSource):
     ''' Maps names of columns to sequences or arrays.
 
-    If the ColumnDataSource initializer is called with a single argument that
-    is a dict or pandas.DataFrame, that argument is used as the value for the
-    "data" attribute. For example::
+    The ``ColumnDataSource`` is a fundamental data structure of Bokeh. Most
+    plots, data tables, etc. will be driven by a ``ColumnDataSource``.
 
-        ColumnDataSource(mydict) # same as ColumnDataSource(data=mydict)
-        ColumnDataSource(df) # same as ColumnDataSource(data=df)
+    If the ColumnDataSource initializer is called with a single argument that
+    can be any of the following:
+
+    * A Python ``dict`` that maps string names to sequences of values, e.g.
+      lists, arrays, etc.
+
+      .. code-block:: python
+
+          data = {'x': [1,2,3,4], 'y': np.ndarray([10.0, 20.0, 30.0, 40.0])}
+
+          source = ColumnDataSource(data)
+
+    * A Pandas ``DataFrame`` object
+
+      .. code-block:: python
+
+          source = ColumnDataSource(df)
+
+      In this case the CDS will have columns corresponding to the columns of
+      the ``DataFrame``. If the ``DataFrame`` has a named index column, then
+      CDS will also have a column with this name. However, if the index name
+      (or any subname of a ``MultiIndex``) is ``None``, then the CDS will have
+      a column generically named ``index`` for the index.
+
+    * A Pandas ``GroupBy`` object
+
+      .. code-block:: python
+
+          group = df.groupby(('colA', 'ColB'))
+
+      In this case the CDS will have columns corresponding to the result of
+      calling ``group.describe()``. The ``describe`` method generates columns
+      for statistical measures such as ``mean`` and ``count`` for all the
+      non-grouped orginal columns. The CDS columns are formed by joining
+      original column names with the computed measure. For example, if a
+      ``DataFrame`` has columns ``'year'`` and ``'mpg'``. Then passing
+      ``df.groupby('year')`` to a CDS will result in columns such as
+      ``'mpg_mean'``
+
+      If the ``GroupBy.describe`` result has a named index column, then
+      CDS will also have a column with this name. However, if the index name
+      (or any subname of a ``MultiIndex``) is ``None``, then the CDS will have
+      a column generically named ``index`` for the index.
+
+      Note this capability to adapt ``GroupBy`` objects may only work with
+      Pandas ``>=0.20.0``.
 
     .. note::
-        There is an implicit assumption that all the columns in a
-        a given ColumnDataSource have the same length.
+        There is an implicit assumption that all the columns in a given
+        ``ColumnDataSource`` all have the same length at all times. For this
+        reason, it is usually preferable to update the ``.data`` property
+        of a data source "all at once".
 
     '''
 
@@ -104,6 +150,8 @@ class ColumnDataSource(ColumnarDataSource):
         if not isinstance(raw_data, dict):
             if pd and isinstance(raw_data, pd.DataFrame):
                 raw_data = self._data_from_df(raw_data)
+            elif pd and isinstance(raw_data, pd.core.groupby.GroupBy):
+                raw_data = self._data_from_groupby(raw_data)
             else:
                 raise ValueError("expected a dict or pandas.DataFrame, got %s" % raw_data)
         super(ColumnDataSource, self).__init__(**kw)
@@ -124,16 +172,42 @@ class ColumnDataSource(ColumnarDataSource):
         '''
         _df = df.copy()
         index = _df.index
-        new_data = _df.to_dict('series')
+        tmp_data = {c: v.values for c, v in _df.iteritems()}
+
+        new_data = {}
+        for k, v in tmp_data.items():
+            if isinstance(k, tuple):
+                k = "_".join(k)
+            new_data[k] = v
 
         if index.name:
             new_data[index.name] = index.values
-        elif index.names and not all([x is None for x in index.names]):
-            new_data["_".join(index.names)] = index.values
+        elif index.names:
+            try:
+                new_data["_".join(index.names)] = index.values
+            except TypeError:
+                new_data["index"] = index.values
         else:
             new_data["index"] = index.values
         return new_data
 
+    @staticmethod
+    def _data_from_groupby(group):
+        ''' Create a ``dict`` of columns from a Pandas GroupBy,
+        suitable for creating a ColumnDataSource.
+
+        The data generated is the result of running ``describe``
+        on the group.
+
+        Args:
+            group (GroupBy) : data to convert
+
+        Returns:
+            dict[str, np.array]
+
+        '''
+        return ColumnDataSource._data_from_df(group.describe())
+
     @classmethod
     def from_df(cls, data):
         ''' Create a ``dict`` of columns from a Pandas DataFrame,
@@ -148,6 +222,23 @@ class ColumnDataSource(ColumnarDataSource):
         '''
         return cls._data_from_df(data)
 
+    @classmethod
+    def from_groupby(cls, data):
+        ''' Create a ``dict`` of columns from a Pandas GroupBy,
+        suitable for creating a ColumnDataSource.
+
+        The data generated is the result of running ``describe``
+        on the group.
+
+        Args:
+            data (Groupby) : data to convert
+
+        Returns:
+            dict[str, np.array]
+
+        '''
+        return cls._data_from_df(data.describe())
+
     def to_df(self):
         ''' Convert this data source to pandas dataframe.
 
@@ -208,7 +299,7 @@ class ColumnDataSource(ColumnarDataSource):
             import warnings
             warnings.warn("Unable to find column '%s' in data source" % name)
 
-    def stream(self, new_data, rollover=None, setter=None):
+    def stream(self, new_data, rollover=None):
         ''' Efficiently update data source columns with new append-only data.
 
         In cases where it is necessary to update data columns in, this method
@@ -247,9 +338,66 @@ class ColumnDataSource(ColumnarDataSource):
             source.stream(new_data)
 
         '''
-        import numpy as np
+        # calls internal implementation
+        self._stream(new_data, rollover)
 
-        newkeys = set(new_data.keys())
+    def _stream(self, new_data, rollover=None, setter=None):
+        ''' Internal implementation to efficiently update data source columns
+        with new append-only data.   The interal implementation adds the setter
+        attribute.  [https://github.com/bokeh/bokeh/issues/6577]
+
+        In cases where it is necessary to update data columns in, this method
+        can efficiently send only the new data, instead of requiring the
+        entire data set to be re-sent.
+
+        Args:
+            new_data (dict[str, seq] or DataFrame or Series) : a mapping of
+                column names to sequences of new data to append to each column,
+                a pandas DataFrame, or a pandas Series in case of a single row -
+                in this case the Series index is used as column names
+
+                All columns of the data source must be present in ``new_data``,
+                with identical-length append data.
+
+            rollover (int, optional) : A maximum column size, above which data
+                from the start of the column begins to be discarded. If None,
+                then columns will continue to grow unbounded (default: None)
+            setter (ClientSession or ServerSession or None, optional) :
+                This is used to prevent "boomerang" updates to Bokeh apps.
+                (default: None)
+                In the context of a Bokeh server application, incoming updates
+                to properties will be annotated with the session that is
+                doing the updating. This value is propagated through any
+                subsequent change notifications that the update triggers.
+                The session can compare the event setter to itself, and
+                suppress any updates that originate from itself.
+        Returns:
+            None
+
+        Raises:
+            ValueError
+
+        Example:
+
+        .. code-block:: python
+
+            source = ColumnDataSource(data=dict(foo=[], bar=[]))
+
+            # has new, identical-length updates for all columns in source
+            new_data = {
+                'foo' : [10, 20],
+                'bar' : [100, 200],
+            }
+
+            source.stream(new_data)
+
+        '''
+        if pd and isinstance(new_data, pd.Series):
+            new_data = new_data.to_frame().T
+        if pd and isinstance(new_data, pd.DataFrame):
+            newkeys = set(new_data.columns)
+        else:
+            newkeys = set(new_data.keys())
         oldkeys = set(self.data.keys())
         if newkeys != oldkeys:
             missing = oldkeys - newkeys
@@ -263,17 +411,21 @@ class ColumnDataSource(ColumnarDataSource):
             else:
                 raise ValueError("Must stream updates to all existing columns (extra: %s)" % ", ".join(sorted(extra)))
 
-        lengths = set()
-        for x in new_data.values():
-            if isinstance(x, np.ndarray):
-                if len(x.shape) != 1:
-                    raise ValueError("stream(...) only supports 1d sequences, got ndarray with size %r" % (x.shape,))
-                lengths.add(x.shape[0])
-            else:
-                lengths.add(len(x))
+        if not (pd and isinstance(new_data, pd.DataFrame)):
+            import numpy as np
 
-        if len(lengths) > 1:
-            raise ValueError("All streaming column updates must be the same length")
+            lengths = set()
+            arr_types = (np.ndarray, pd.Series) if pd else np.ndarray
+            for k, x in new_data.items():
+                if isinstance(x, arr_types):
+                    if len(x.shape) != 1:
+                        raise ValueError("stream(...) only supports 1d sequences, got ndarray with size %r" % (x.shape,))
+                    lengths.add(x.shape[0])
+                else:
+                    lengths.add(len(x))
+
+            if len(lengths) > 1:
+                raise ValueError("All streaming column updates must be the same length")
 
         self.data._stream(self.document, self, new_data, rollover, setter)
 
@@ -442,6 +594,20 @@ def _check_slice(s):
        (s.step  is not None and s.step < 1):
         raise ValueError("Patch slices must have positive (start, stop, step) values, got %s" % s)
 
+class CDSView(Model):
+    ''' A view into a ColumnDataSource that represents a row-wise subset.
+
+    '''
+
+    filters = List(Instance(Filter), default=[], help="""
+    List of filters that the view comprises.
+    """)
+
+    source = Instance(ColumnarDataSource, help="""
+    The ColumnDataSource associated with this view. Used to determine
+    the length of the columns.
+    """)
+
 class GeoJSONDataSource(ColumnarDataSource):
     '''
 
