@@ -67,10 +67,10 @@ The inline example code above produces the following output:
 from __future__ import absolute_import
 
 import ast
-import hashlib
 from os import getenv
 from os.path import basename, dirname, join
 import re
+from uuid import uuid4
 
 from docutils import nodes
 from docutils.parsers.rst import Directive, Parser
@@ -88,11 +88,26 @@ from ..util.string import decode_utf8
 from .example_handler import ExampleHandler
 from .templates import PLOT_PAGE
 
-if settings.docs_cdn() == "local":
-    resources = Resources(mode="server", root_url="/en/latest/")
-else:
+docs_cdn = settings.docs_cdn()
+
+# if BOKEH_DOCS_CDN is unset just use default CDN resources
+if docs_cdn is None:
     resources = Resources(mode="cdn")
 
+else:
+    # "BOKEH_DOCS_CDN=local" is used for building and displaying the docs locally
+    if docs_cdn == "local":
+        resources = Resources(mode="server", root_url="/en/latest/")
+
+    # "BOKEH_DOCS_CDN=test:newthing" is used for building and deploying test docs to
+    # a one-off location "en/newthing" on the docs site
+    elif docs_cdn.startswith("test:"):
+        resources = Resources(mode="server", root_url="/en/%s/" % docs_cdn.split(":")[1])
+
+    # Otherwise assume it is a dev/rc/full release version and use CDN for it
+    else:
+        resources = Resources(mode="cdn", version=docs_cdn)
+
 GOOGLE_API_KEY = getenv('GOOGLE_API_KEY')
 if GOOGLE_API_KEY is None:
     if settings.docs_missing_api_key_ok():
@@ -163,7 +178,7 @@ class PlotScriptParser(Parser):
             lineno = m.body[0].lineno # assumes docstring is m.body[0]
             source = "\n".join(lines[lineno:])
 
-        js_name = "bokeh-plot-%s.js" % hashlib.md5(env.docname.encode('utf-8')).hexdigest()
+        js_name = "bokeh-plot-%s.js" % uuid4().hex
 
         (script, js, js_path, source) = _process_script(source, filename, env.bokeh_plot_auxdir, js_name)
 
@@ -204,8 +219,7 @@ class BokehPlotDirective(Directive):
             source = '\n'.join(self.content)
             # need docname not to look like a path
             docname = env.docname.replace("/", "-")
-            serialno = env.new_serialno(env.docname)
-            js_name = "bokeh-plot-%s-inline-%d.js" % (docname, serialno)
+            js_name = "bokeh-plot-%s-inline-%s.js" % (docname, uuid4().hex)
             # the code runner just needs a real path to cd to, this will do
             path = join(env.bokeh_plot_auxdir, js_name)
 
@@ -227,8 +241,7 @@ class BokehPlotDirective(Directive):
                 source = open(self.arguments[0]).read()
                 source = decode_utf8(source)
                 docname = env.docname.replace("/", "-")
-                serialno = env.new_serialno(env.docname)
-                js_name = "bokeh-plot-%s-external-%d.js" % (docname, serialno)
+                js_name = "bokeh-plot-%s-external-%s.js" % (docname, uuid4().hex)
                 (script, js, js_path, source) = _process_script(source, self.arguments[0], env.bokeh_plot_auxdir, js_name)
                 env.bokeh_plot_files[js_name] = (script, js, js_path, source)
 
